/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.htrace.core;

import static org.junit.Assert.assertTrue;
import org.junit.Test;

public class TestTracerPool {
  HTraceConfiguration conf = HTraceConfiguration.EMPTY;
  ClassLoader loader = TestTracerPool.class.getClassLoader();

  @Test
  public void testLoadReceiver() {
    addThenLoadReceiverType("org.apache.htrace.core.POJOSpanReceiver");
  }

  @Test
  public void testLoadReceiverWithSimpleName() {
    addThenLoadReceiverType("POJOSpanReceiver");
  }

  public void addThenLoadReceiverType(String className) {
    TracerPool pool = new TracerPool(className);
    SpanReceiver receiver = new SpanReceiver.Builder(conf).
        className(className).
        build();
    pool.addReceiver(receiver);
    assertTrue(receiver == pool.loadReceiverType(className, conf, loader));
  }
}
