/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.htrace.core;

import org.junit.Assert;
import org.junit.Test;

public class TestNullScope {
  private void verifyNullScope(TraceScope scope) {
    Assert.assertTrue(null == scope.getSpan());
    Assert.assertFalse(scope.detached);
    scope.detach();
    Assert.assertTrue(scope.detached);
    scope.reattach();
    Assert.assertFalse(scope.detached);
    scope.close();
  }

  @Test
  public void testNullScope() {
    Tracer tracer = new Tracer.Builder().
        name("testNullScope").
        tracerPool(new TracerPool("testNullScope")).
        conf(HTraceConfiguration.EMPTY).
        build();
    verifyNullScope(tracer.newScope("testNullScope"));
    verifyNullScope(tracer.newNullScope());
  }
}
