/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

option java_package = "org.apache.htrace.protobuf.generated";
option java_outer_classname = "SpanProtos";
option java_generate_equals_and_hash = true;
option optimize_for = SPEED;

message TimelineAnnotation {
  required int64 time = 1;
  required string message = 2;
}

message Span {
  required int64 trace_id = 1;
  required int64 parent_id = 2;
  required int64 start = 3;
  required int64 stop = 4;
  required int64 span_id = 5;
  required string process_id = 6;
  required string description = 7;
  repeated TimelineAnnotation timeline = 8;
}
