/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package common

import (
	"testing"
)

func TestValidOps(t *testing.T) {
	for i := range ValidOps() {
		op := ValidOps()[i]
		if !op.IsValid() {
			t.Fatalf("op %s was in ValidOps, but IsValid returned false.\n", op)
		}
	}
	invalidOp := Op("completelybogus")
	if invalidOp.IsValid() {
		t.Fatalf("op %s was invalid, but IsValid returned true.\n", invalidOp)
	}
}

func TestValidFields(t *testing.T) {
	for i := range ValidFields() {
		field := ValidFields()[i]
		if !field.IsValid() {
			t.Fatalf("field %s was in ValidFields, but IsValid returned false.\n", field)
		}
	}
	invalidField := Field("completelybogus")
	if invalidField.IsValid() {
		t.Fatalf("field %s was invalid, but IsValid returned true.\n", invalidField)
	}
}
